#!/usr/bin/env node

const path = require('path');
const fs = require('fs').promises;
const backstop = require('backstopjs');
const chalk = require('chalk');
const argv = require('yargs').argv;
const CONFIG = require('./config');

/* eslint-disable-next-line @typescript-eslint/naming-convention */
const { S3ClientWrapper } = require('./s3client');

async function parseOptions() {
  // Load the config file.
  const configFile = argv?.config || '.vregrc.json';
  try {
    await fs.stat(configFile);
  }
  catch {
    console.log(chalk.red(`Could not load config file from ${ configFile }. Please check that the file exists.`));
    process.exit(1);
  }

  const config = JSON.parse(await fs.readFile(configFile));
  const viewports = config.viewports.map((v) => typeof v === 'string' ? CONFIG.STANDARD_VIEWPORTS[v] : v);
  const basePath = argv?.path || process.cwd();

  return {
    ...config,
    path: basePath,
    viewports,
  };
}

async function* findScenarios(dir) {
  const entries = await fs.readdir(dir, { withFileTypes: true });
  for (const entry of entries) {
    const res = path.resolve(dir, entry.name);
    if (entry.isDirectory()) {
      yield* findScenarios(res);
    }
    else if (entry.name.match(/.*\.vreg.js$/)) {
      yield res;
    }
  }
}

async function loadScenarios(options) {
  const scenarios = [];
  for await (const file of findScenarios(options.path)) {
    scenarios.push(...require(file).map((x) => {
      const scenario = { ...CONFIG.TEST_DEFAULTS, ...x };
      if (!scenario.url.match(/^http[s]?:\/\//)) {
        scenario.url = `${ options.baseUrl }${ scenario.url }`;
      }
      return scenario;
    }));
  }
  return scenarios;
}

async function buildConfig(options) {
  return {
    id: options.storageKey,
    viewports: options.viewports,
    scenarios: await loadScenarios(options),
    paths: {
      /* eslint-disable @typescript-eslint/naming-convention */
      'bitmaps_reference': CONFIG.DATA_REFERENCE_DIR,
      'bitmaps_test': CONFIG.DATA_TEST_DIR,
      'engine_scripts': CONFIG.DATA_ENGINE_SCRIPTS_DIR,
      'html_report': CONFIG.DATA_HTML_REPORT_DIR,
      'ci_report': CONFIG.DATA_CI_REPORT_DIR,
      /* eslint-enable @typescript-eslint/naming-convention */
    },
    report: ['browser', 'CI'],
    engine: 'puppeteer',
    engineFlags: ['--no-sandbox'],
    ci: {
      format: 'junit',
      testSuiteName: `${ options.project } Visual Regression`,
    },
    asyncCaptureLimit: 5,
    asyncCompareLimit: 50,
    debug: false,
    debugWindow: false,
  };
}

async function run(command) {
  const options = await parseOptions();
  const config = await buildConfig(options);
  const s3 = new S3ClientWrapper(options);

  console.log(chalk.bold(`Loaded ${ config.scenarios.length } scenarios.`));

  if (command === 'test') {
    await s3.pullReferenceSet();
  }

  await backstop(command, { config });

  if (command === 'approve') {
    await s3.pushReferenceSet();
  }
}

module.exports = run;
